package im.shimo.react.textInput;

import android.content.Context;
import android.view.KeyEvent;
import android.view.View;
import android.view.inputmethod.EditorInfo;
import android.view.inputmethod.InputConnection;
import android.view.inputmethod.InputConnectionWrapper;

import com.facebook.react.views.textinput.ReactEditText;

public class RNEditText extends ReactEditText {
    private boolean mShouldReturn = true;
    private EnventInterpolater enventInterpolater;

    public RNEditText(Context context) {
        super(context);
        //长按调用的是TextView.performLongClick()
        //然后定位到Editor.prepareCursorControllers()
        // ，发现关键是mInsertionControllerEnabled为false
        //所以定位到问题所在：还没有attachedToWindow或者Cursor还没有显示时，此值不可能是true
        //解决办法就是在EditText添加到窗口后，直接调用setCursorVisible方法
        //先false后true，让方法改变mInsertionControllerEnabled的值为true
        //目前还不清楚是不是所有版本的系统都会有这个bug 所以不用区分系统版本
        addOnAttachStateChangeListener(new View.OnAttachStateChangeListener() {
            @Override
            public void onViewAttachedToWindow(View v) {
                setCursorVisible(false);
                setCursorVisible(true);
            }

            @Override
            public void onViewDetachedFromWindow(View v) {
            }
        });
    }

    @Override
    public InputConnection onCreateInputConnection(EditorInfo outAttrs) {
        final InputConnection connection = super.onCreateInputConnection(outAttrs);
        //父类将EditorInfo的值给覆盖了，如果想要横屏时键盘不是全屏模式，必须加上这个方法的重写
        outAttrs.imeOptions |= EditorInfo.IME_FLAG_NO_EXTRACT_UI;
        if (connection != null) {
            //事件拦截处理
            InputConnectionWrapper inputConnectionWrapper = null;
            if (enventInterpolater != null) {
                inputConnectionWrapper = new InputConnectionWrapper(
                        connection, true) {
                    @Override
                    public boolean sendKeyEvent(KeyEvent event) {
                        if (!enventInterpolater.sendEvent(RNEditText.this, event)) {
                            return super.sendKeyEvent(event);
                        } else {
                            return true;
                        }
                    }

                    @Override
                    public boolean finishComposingText() {
                        try {
                            return super.finishComposingText();
                        } catch (Exception e) {
                            return true;
                        } finally {
                            if (connection != null) {
                                return connection.finishComposingText();
                            }
                        }
                    }
                };
                return inputConnectionWrapper;
            }
        }
        return connection;
    }

    public void setShouldReturn(boolean shouldReturn) {
        mShouldReturn = shouldReturn;
    }

    public boolean getShouldReturn() {
        return mShouldReturn;
    }

    @Override
    public boolean isLayoutRequested() {
        return false;
    }

    public void setEnventInterpolater(EnventInterpolater enventInterpolater) {
        this.enventInterpolater = enventInterpolater;
    }
}
