package im.shimo.react.textInput;

import android.text.InputType;
import android.util.TypedValue;
import android.view.KeyEvent;
import android.widget.EditText;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.module.annotations.ReactModule;
import com.facebook.react.uimanager.PixelUtil;
import com.facebook.react.uimanager.ThemedReactContext;
import com.facebook.react.uimanager.UIManagerModule;
import com.facebook.react.uimanager.ViewDefaults;
import com.facebook.react.uimanager.annotations.ReactProp;
import com.facebook.react.uimanager.events.Event;
import com.facebook.react.uimanager.events.EventDispatcher;
import com.facebook.react.uimanager.events.RCTEventEmitter;
import com.facebook.react.views.textinput.ReactEditText;
import com.facebook.react.views.textinput.ReactTextInputManager;


@ReactModule(name = RNTextInputManager.REACT_CLASS)
public class RNTextInputManager extends ReactTextInputManager implements EnventInterpolater {
    protected static final String REACT_CLASS = "RNTextInput";

    private class ReactTextInputSubmitEditingEvent
            extends Event<ReactTextInputSubmitEditingEvent> {

        private static final String EVENT_NAME = "topSubmitEditing";

        private String mText;

        public ReactTextInputSubmitEditingEvent(
                int viewId,
                String text) {
            super(viewId);
            mText = text;
        }

        @Override
        public String getEventName() {
            return EVENT_NAME;
        }

        @Override
        public boolean canCoalesce() {
            return false;
        }

        @Override
        public void dispatch(RCTEventEmitter rctEventEmitter) {
            rctEventEmitter.receiveEvent(getViewTag(), getEventName(), serializeEventData());
        }

        private WritableMap serializeEventData() {
            WritableMap eventData = Arguments.createMap();
            eventData.putInt("target", getViewTag());
            eventData.putString("text", mText);
            return eventData;
        }

    }

    @Override
    public String getName() {
        return REACT_CLASS;
    }

    @Override
    public ReactEditText createViewInstance(ThemedReactContext context) {
        RNEditText editText = new RNEditText(context);
        int inputType = editText.getInputType();
        editText.setInputType(inputType & (~InputType.TYPE_TEXT_FLAG_MULTI_LINE));
        editText.setReturnKeyType("done");
        editText.setTextSize(
                TypedValue.COMPLEX_UNIT_PX,
                (int) Math.ceil(PixelUtil.toPixelFromSP(ViewDefaults.FONT_SIZE_SP)));
        editText.setEnventInterpolater(this);
        return editText;
    }

    @ReactProp(name = "shouldReturn")
    public void setShouldReturn(RNEditText editText, boolean shouldReturn) {
        editText.setShouldReturn(shouldReturn);
    }

    @ReactProp(name = "focusable", defaultBoolean = true)
    public void setFocusable(RNEditText editText, boolean focusable) {
        editText.setFocusable(focusable);
    }

    @Override
    protected void addEventEmitters(
            final ThemedReactContext reactContext,
            final ReactEditText editText) {
        super.addEventEmitters(reactContext, editText);
    }

    @Override
    public boolean sendEvent(EditText editText, KeyEvent event) {
        if (event.getKeyCode() == KeyEvent.KEYCODE_ENTER) {
            boolean blurOnSubmit = ((RNEditText) editText).getBlurOnSubmit();
            boolean isMultiline = ((editText.getInputType() &
                    InputType.TYPE_TEXT_FLAG_MULTI_LINE) != 0);
            boolean shouldReturn = ((RNEditText) editText).getShouldReturn();
            if (event.getAction() == KeyEvent.ACTION_UP) {//只处理up事件
                // Motivation:
                // * blurOnSubmit && isMultiline => Clear focus; prevent default behaviour (return true);
                // * blurOnSubmit && !isMultiline => Clear focus; prevent default behaviour (return true);
                // * !blurOnSubmit && isMultiline => Perform default behaviour (return false);
                // * !blurOnSubmit && !isMultiline => Prevent default behaviour (return true).
                // Additionally we always generate a `submit` event.

                EventDispatcher eventDispatcher =
                        ((ThemedReactContext) editText.getContext()).getNativeModule(UIManagerModule.class).getEventDispatcher();

                eventDispatcher.dispatchEvent(
                        new ReactTextInputSubmitEditingEvent(
                                editText.getId(),
                                editText.getText().toString()));

                if (blurOnSubmit && shouldReturn) {
                    editText.clearFocus();
                }
            }
            // Prevent default behavior except when we want it to insert a newline.
            return !shouldReturn || blurOnSubmit || !isMultiline;
        } else {
            return false;
        }
    }
}
