//
//  RNMarkedRangeTextField.m
//  RNMarkedRangeTextInput
//
//  Created by Bell Zhong on 2017/5/18.
//  Copyright © 2017年 shimo. All rights reserved.
//

#import "RNMarkedRangeTextField.h"

@implementation RNMarkedRangeTextField


- (instancetype)initWithBridge:(RCTBridge *)bridge
{
    if ((self = [super initWithBridge:bridge])) {
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(textFieldChanged:) name:UITextFieldTextDidChangeNotification object:nil];
    }
    return self;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UITextFieldTextDidChangeNotification object:nil];
}

- (void)textFieldChanged:(NSNotification* )notification
{
    if (notification.object != self.backedTextInputView) {
        return;
    }
    UITextField *textField = (UITextField* )self.backedTextInputView;
    UITextRange *markedRange = textField.markedTextRange;
    NSInteger start = [textField offsetFromPosition:[textField beginningOfDocument] toPosition:markedRange.start];
    NSInteger end = [textField offsetFromPosition:[textField beginningOfDocument] toPosition:markedRange.end];
    NSString *markedText = [textField textInRange:markedRange];
    NSString *text = textField.text;
    if (end > start) {
        NSString *temp = @"";
        if (start > 0) {
            temp = [temp stringByAppendingString:[text substringToIndex:start]];
        }
        NSInteger length = text.length;
        if (end < length) {
            temp = [temp stringByAppendingString:[text substringFromIndex:end]];
        }
        text = temp;
    }
    
    _onMarkedRangeChanged(@{
                            @"markedRange": @{
                                    @"start": @(start),
                                    @"end": @(end),
                                    @"text": markedText
                                    }
                            });
    
    _onChangeText(@{
                    @"text": text,
                    });
}

@end
