//
//  RNMarkedRangeTextView.m
//  RNMarkedRangeTextInput
//
//  Created by Bell Zhong on 2017/5/18.
//  Copyright © 2017年 shimo. All rights reserved.
//

#import "RNMarkedRangeTextView.h"
#import "RCTEventDispatcher.h"
#import "RNMarkedRangeTextField.h"
#import "RCTUITextView.h"

@implementation RNMarkedRangeTextView

- (instancetype)initWithBridge:(RCTBridge *)bridge {
    if ((self = [super initWithBridge:bridge])) {
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(textFieldChanged:) name:UITextViewTextDidChangeNotification object:nil];
    }
    return self;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UITextViewTextDidChangeNotification object:nil];
}

-(void)textFieldWillChange:(NSNotification* )notification
{
    UITextView *textView = nil;
    @try {
        textView = (UITextView*) [self valueForKey:@"_textView"];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception);
    }
    
    if (textView == nil) {
        @try {
            textView = (UITextView*) [self valueForKey:@"_backedTextInput"];
        } @catch (NSException *exception) {
            NSLog(@"%@", exception);
        }
    }
    
    if (!textView || notification.object != textView) {
        return;
    }
    
    
}

- (BOOL)textInputShouldChangeTextInRange:(NSRange)range replacementText:(NSString *)string
{
    UITextView<RCTBackedTextInputViewProtocol> *backedTextInput = nil;
    @try {
        backedTextInput = (RCTUITextView*) [self valueForKey:@"_backedTextInput"];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception);
    }
    
    if (backedTextInput && !backedTextInput.textWasPasted && [string isEqualToString:@"\n"] && !self.shouldReturn) {
        return NO;
    }
    
    return YES;
}

- (void)textFieldChanged:(NSNotification* )notification
{
    RCTUITextView *textView = nil;
    @try {
        textView = (RCTUITextView*) [self valueForKey:@"_backedTextInput"];
    } @catch (NSException *exception) {
        NSLog(@"%@", exception);
    }
    if (!textView || notification.object != textView) {
        return;
    }
    UITextRange *markedRange = textView.markedTextRange;
    NSString * markedText = [textView textInRange:markedRange];
    NSInteger start = [textView offsetFromPosition:[textView beginningOfDocument] toPosition:markedRange.start];
    NSInteger end = [textView offsetFromPosition:[textView beginningOfDocument] toPosition:markedRange.end];
    NSString *text = textView.text;
    if (end > start) {
        NSString *temp = @"";
        if (start > 0) {
            temp = [temp stringByAppendingString:[text substringToIndex:start]];
        }
        NSInteger length = text.length;
        if (end < length) {
            temp = [temp stringByAppendingString:[text substringFromIndex:end]];
        }
        text = temp;
    }
    _onMarkedRangeChanged(@{
                            @"markedRange": @{
                                    @"start": @(start),
                                    @"end": @(end),
                                    @"text": markedText
                                    },
                            });
    _onChangeText(@{
                    @"text": text,
                    });
}

@end
